@@ -61,7 +61,31 @@ astrometric_cuts = (BJ.r_lo <= 1500,
                        fn.pow(epar, 2) / (4.5 * 4.5) <= 1)))
 
 
-class MWM_CB_UVEX1_Carton(BaseCarton):
+class MWM_CB_UVEX_BaseCarton(BaseCarton):
+
+    def post_process(self, model, **kwargs):
+
+        self.database.execute_sql(f'ALTER TABLE {self.path} ADD COLUMN value REAL;')
+        model._meta.add_field('value', peewee.FloatField())
+
+        # G < 17 => cadence = bright_1x1
+        model.update(cadence='bright_1x1').where(model.phot_g_mean_mag < 17).execute()
+        model.update(value=1).where(model.phot_g_mean_mag < 17).execute()
+
+        # 17 < G < 19 => cadence = dark_1x2
+        model.update(cadence='dark_1x2').where((model.phot_g_mean_mag > 17) &
+                                               (model.phot_g_mean_mag < 19)).execute()
+        model.update(value=2).where((model.phot_g_mean_mag > 17) &
+                                    (model.phot_g_mean_mag < 19)).execute()
+
+        # G > 19 => cadence = dark_1x3
+        model.update(cadence='dark_1x3').where(model.phot_g_mean_mag > 19).execute()
+        model.update(value=3).where(model.phot_g_mean_mag > 19).execute()
+
+        return model
+
+
+class MWM_CB_UVEX1_Carton(MWM_CB_UVEX_BaseCarton):
     """MWM Compact Binaries UV excess 1.
 
     Definition:
@@ -119,9 +143,8 @@ class MWM_CB_UVEX1_Carton(BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    # mwm_cb_2x1 -> boss_dark_2x1
     instrument = 'BOSS'
-    cadence = 'dark_2x1'
+    cadence = None
     priority = 1400
 
     def setup_transaction(self):
@@ -191,7 +214,7 @@ class MWM_CB_UVEX1_Carton(BaseCarton):
         return query
 
 
-class MWM_CB_UVEX2_Carton(BaseCarton):
+class MWM_CB_UVEX2_Carton(MWM_CB_UVEX_BaseCarton):
     """MWM Compact Binaries UV excess 2.
 
     Definition:
@@ -251,9 +274,8 @@ class MWM_CB_UVEX2_Carton(BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    # mwm_cb_2x1 -> boss_dark_2x1
     instrument = 'BOSS'
-    cadence = 'dark_2x1'
+    cadence = None
     priority = 1400
 
     def setup_transaction(self):
@@ -317,7 +339,7 @@ class MWM_CB_UVEX2_Carton(BaseCarton):
         return query
 
 
-class MWM_CB_UVEX3_Carton(BaseCarton):
+class MWM_CB_UVEX3_Carton(MWM_CB_UVEX_BaseCarton):
     """MWM Compact Binaries UV excess 3.
 
     Definition:
@@ -332,30 +354,14 @@ class MWM_CB_UVEX3_Carton(BaseCarton):
         - Apply some general UV quality flags and work on subset of entries
           in OM-SUSS:
             - (UVW2_AB_MAG >-100) || (UVW1_AB_MAG >-100) || (UVM2_AB_MAG>-100)
-            - !(equals(substring(UVW2_QUALITY_FLAG_ST,0,1),"T") ||
-                equals(substring(UVM2_QUALITY_FLAG_ST,0,1),"T") ||
-                equals(substring(UVW1_QUALITY_FLAG_ST,0,1),"T") ||
-                (equals(substring(UVW2_QUALITY_FLAG_ST,1,2),"T") &&
-                 UVW2_SIGNIF <10) ||
+            - !(equals(substring(UVM2_QUALITY_FLAG_ST,0,1),"T") ||
                 (equals(substring(UVM2_QUALITY_FLAG_ST,1,2),"T") &&
                  UVM2_SIGNIF <10) ||
-                (equals(substring(UVW1_QUALITY_FLAG_ST,1,2),"T") &&
-                 UVW1_SIGNIF <10) ||
-                (equals(substring(UVW2_QUALITY_FLAG_ST,2,3) ,"T") &&
-                 UVW2_SIGNIF <10) ||
                 (equals(substring(UVM2_QUALITY_FLAG_ST,2,3),"T") &&
                  UVM2_SIGNIF <10) ||
-                (equals(substring(UVW1_QUALITY_FLAG_ST,2,3),"T") &&
-                 UVW1_SIGNIF <10) ||
-                (equals(substring(UVW2_QUALITY_FLAG_ST,6,7) ,"T")) ||
                 equals(substring(UVM2_QUALITY_FLAG_ST,6,7),"T") ||
-                equals(substring(UVW1_QUALITY_FLAG_ST,6,7),"T") ||
-                equals(substring(UVW2_QUALITY_FLAG_ST,7,8),"T") ||
                 equals(substring(UVM2_QUALITY_FLAG_ST,7,8),"T") ||
-                equals(substring(UVW1_QUALITY_FLAG_ST,7,8),"T") ||
-                equals(substring(UVW2_QUALITY_FLAG_ST,8,9),"T") ||
-                equals(substring(UVM2_QUALITY_FLAG_ST,8,9),"T") ||
-                equals(substring(UVW1_QUALITY_FLAG_ST,8,9),"T"))
+                equals(substring(UVM2_QUALITY_FLAG_ST,8,9),"T"))
 
         - Match UV catalog with Gaia DR2 within 3 arcsec, keep the nearest
           match only, add distance and lower limit distance (columns r_est,
@@ -400,45 +406,24 @@ class MWM_CB_UVEX3_Carton(BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    # mwm_cb_2x1 -> boss_dark_2x1
     instrument = 'BOSS'
-    cadence = 'dark_2x1'
+    cadence = None
     priority = 1400
 
     def build_query(self, version_id, query_region=None):
 
-        uvw2_ab_mag = XMM_OM_SUSS_4_1.uvw2_ab_mag
-        uvw1_ab_mag = XMM_OM_SUSS_4_1.uvw1_ab_mag
         uvm2_ab_mag = XMM_OM_SUSS_4_1.uvm2_ab_mag
-
-        uvw2_quality_flag_st = XMM_OM_SUSS_4_1.uvw2_quality_flag_st
         uvm2_quality_flag_st = XMM_OM_SUSS_4_1.uvm2_quality_flag_st
-        uvw1_quality_flag_st = XMM_OM_SUSS_4_1.uvw1_quality_flag_st
-
-        uvw2_signif = XMM_OM_SUSS_4_1.uvw2_signif
         uvm2_signif = XMM_OM_SUSS_4_1.uvm2_signif
-        uvw1_signif = XMM_OM_SUSS_4_1.uvw1_signif
 
         quality_flags = (
-            (uvw2_ab_mag > -100) | (uvw1_ab_mag > -100) | (uvm2_ab_mag > -100),
-            ~((fn.substr(uvw2_quality_flag_st, 1, 1) == 'T') |
-              (fn.substr(uvm2_quality_flag_st, 1, 1) == 'T') |
-              (fn.substr(uvw1_quality_flag_st, 1, 1) == 'T') |
-              ((fn.substr(uvw2_quality_flag_st, 2, 1) == 'T') & (uvw2_signif < 10)) |
+            (uvm2_ab_mag > -100),
+            ~((fn.substr(uvm2_quality_flag_st, 1, 1) == 'T') |
               ((fn.substr(uvm2_quality_flag_st, 2, 1) == 'T') & (uvm2_signif < 10)) |
-              ((fn.substr(uvw1_quality_flag_st, 2, 1) == 'T') & (uvw1_signif < 10)) |
-              ((fn.substr(uvw2_quality_flag_st, 3, 1) == 'T') & (uvw2_signif < 10)) |
               ((fn.substr(uvm2_quality_flag_st, 3, 1) == 'T') & (uvm2_signif < 10)) |
-              ((fn.substr(uvw1_quality_flag_st, 3, 1) == 'T') & (uvw1_signif < 10)) |
-              ((fn.substr(uvw2_quality_flag_st, 7, 1) == 'T') |
-               (fn.substr(uvm2_quality_flag_st, 7, 1) == 'T') |
-               (fn.substr(uvw1_quality_flag_st, 7, 1) == 'T') |
-               (fn.substr(uvw2_quality_flag_st, 8, 1) == 'T') |
-               (fn.substr(uvm2_quality_flag_st, 8, 1) == 'T') |
-               (fn.substr(uvw1_quality_flag_st, 8, 1) == 'T') |
-               (fn.substr(uvw2_quality_flag_st, 9, 1) == 'T') |
-               (fn.substr(uvm2_quality_flag_st, 9, 1) == 'T') |
-               (fn.substr(uvw1_quality_flag_st, 9, 1) == 'T')))
+              (fn.substr(uvm2_quality_flag_st, 7, 1) == 'T') |
+              (fn.substr(uvm2_quality_flag_st, 8, 1) == 'T') |
+              (fn.substr(uvm2_quality_flag_st, 9, 1) == 'T'))
         )
 
         quality_cte = (XMM_OM_SUSS_4_1
@@ -450,10 +435,11 @@ class MWM_CB_UVEX3_Carton(BaseCarton):
                        .where(*quality_flags)
                        .cte('quality_cte', materialized=True))
 
-        colour_cuts = (colour_absg,
-                       ((quality_cte.c.uvm2_ab_mag - gmagab) < 2.25) |
-                       (((quality_cte.c.uvm2_ab_mag - gmagab) < 6) &
-                        ((quality_cte.c.uvm2_ab_mag - gmagab) < 5.57377 * (B_AB_R_AB) + 0.2049)))
+        colour_cuts = (
+            colour_absg,
+            ((quality_cte.c.uvm2_ab_mag - gmagab) < 2.25) |
+            (((quality_cte.c.uvm2_ab_mag - gmagab) < 6) &
+             ((quality_cte.c.uvm2_ab_mag - gmagab) < 5.57377 * (B_AB_R_AB) + 0.2049)))
 
         query = (Catalog
                  .select(Catalog.catalogid,
@@ -472,13 +458,13 @@ class MWM_CB_UVEX3_Carton(BaseCarton):
                          gmagab.alias('gmagab'),
                          BJ.r_est,
                          BJ.r_lo,
-                         XMM_OM_SUSS_4_1.srcnum,
+                         quality_cte.c.srcnum,
+                         quality_cte.c.obsid,
+                         quality_cte.c.ra.alias('xmm_ra'),
+                         quality_cte.c.dec.alias('xmm_dec'),
+                         quality_cte.c.uvm2_signif,
                          quality_cte.c.uvm2_ab_mag.alias('uvm2_ab_mag'),
-                         quality_cte.c.uvw2_ab_mag.alias('uvw2_ab_mag'),
-                         quality_cte.c.uvw1_ab_mag.alias('uvw1_ab_mag'),
-                         quality_cte.c.uvm2_quality_flag_st.alias('uvm2_quality_flag_st'),
-                         quality_cte.c.uvw2_quality_flag_st.alias('uvw2_quality_flag_st'),
-                         quality_cte.c.uvw1_quality_flag_st.alias('uvw1_quality_flag_st'))
+                         quality_cte.c.uvm2_quality_flag_st.alias('uvm2_quality_flag_st'))
                  .join(CatalogToTIC_v8)
                  .join(TIC_v8)
                  .join(Gaia_DR2)
@@ -493,17 +479,16 @@ class MWM_CB_UVEX3_Carton(BaseCarton):
                  .with_cte(quality_cte))
 
         if query_region:
-            query = (query
-                     .where(peewee.fn.q3c_radial_query(Catalog.ra,
-                                                       Catalog.dec,
-                                                       query_region[0],
-                                                       query_region[1],
-                                                       query_region[2])))
+            query = (query.where(peewee.fn.q3c_radial_query(Catalog.ra,
+                                                            Catalog.dec,
+                                                            query_region[0],
+                                                            query_region[1],
+                                                            query_region[2])))
 
         return query
 
 
-class MWM_CB_UVEX4_Carton(BaseCarton):
+class MWM_CB_UVEX4_Carton(MWM_CB_UVEX_BaseCarton):
     """MWM Compact Binaries UV excess 4.
 
     Definition:
@@ -518,20 +503,10 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
         - Apply some general UV quality flags and work on subset of entries
         in UVOT catalog::
             - (UVW2_AB_MAG >-100) || (UVW1_AB_MAG >-100) || (UVM2_AB_MAG>-100)
-            - !(equals(substring(UVW1_QUALITY_FLAG_BIN,8,9),"1") ||
-                equals(substring(UVW2_QUALITY_FLAG_BIN,8,9),"1") ||
-                equals(substring(UVM2_QUALITY_FLAG_BIN,8,9),"1") ||
-                (equals(substring(UVW1_QUALITY_FLAG_BIN,7,8),"1") && UVW1_SIGNIF <10) ||
-                (equals(substring(UVW2_QUALITY_FLAG_BIN,7,8),"1") && UVW2_SIGNIF <10) ||
+            - !(equals(substring(UVM2_QUALITY_FLAG_BIN,8,9),"1") ||
                 (equals(substring(UVM2_QUALITY_FLAG_BIN,7,8),"1") && UVM2_SIGNIF <10) ||
-                (equals(substring(UVW1_QUALITY_FLAG_BIN,6,7),"1") && UVW1_SIGNIF <10) ||
-                (equals(substring(UVW2_QUALITY_FLAG_BIN,6,7),"1") && UVW2_SIGNIF <10) ||
                 (equals(substring(UVM2_QUALITY_FLAG_BIN,6,7),"1") && UVM2_SIGNIF <10) ||
-                (equals(substring(UVW1_QUALITY_FLAG_BIN,3,4),"1") && UVW1_SIGNIF <10) ||
-                (equals(substring(UVW2_QUALITY_FLAG_BIN,3,4),"1") && UVW2_SIGNIF <10) ||
                 (equals(substring(UVM2_QUALITY_FLAG_BIN,3,4),"1") && UVM2_SIGNIF <10) ||
-                equals(substring(UVW1_QUALITY_FLAG_BIN,2,3),"1") ||
-                equals(substring(UVW2_QUALITY_FLAG_BIN,2,3),"1") ||
                 equals(substring(UVM2_QUALITY_FLAG_BIN,2,3),"1"))
 
         - Match UV catalog with Gaia DR2 within 3 arcsec, keep the nearest
@@ -577,9 +552,8 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    # mwm_cb_2x1 -> boss_dark_2x1
     instrument = 'BOSS'
-    cadence = 'dark_2x1'
+    cadence = None
     priority = 1400
 
     def build_query(self, version_id, query_region=None):
@@ -588,33 +562,18 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
         uvm2_ab_mag = UVOT_SSC_1.uvm2_ab
         uvw1_ab_mag = UVOT_SSC_1.uvw1_ab
 
-        uvw2_quality_flag = UVOT_SSC_1.fuvw2
         uvm2_quality_flag = UVOT_SSC_1.fuvm2
-        uvw1_quality_flag = UVOT_SSC_1.fuvw1
-
-        uvw2_signif = UVOT_SSC_1.suvw2
         uvm2_signif = UVOT_SSC_1.suvm2
-        uvw1_signif = UVOT_SSC_1.suvw1
 
         # uvXY_quality_flag is a 9-bit integer so character 8 (0-indexed) in
         # the binary string is the first bit, 7 is the second, etc.
 
         quality_flags = (
             (uvw2_ab_mag > -100) | (uvw1_ab_mag > -100) | (uvm2_ab_mag > -100),
-            ~((uvw1_quality_flag.bin_and(2**0) > 0) |
-              (uvw2_quality_flag.bin_and(2**0) > 0) |
-              (uvm2_quality_flag.bin_and(2**0) > 0) |
-              (uvw1_quality_flag.bin_and(2**1) > 0) & (uvw1_signif < 10) |
-              (uvw2_quality_flag.bin_and(2**1) > 0) & (uvw2_signif < 10) |
+            ~((uvm2_quality_flag.bin_and(2**0) > 0) |
               (uvm2_quality_flag.bin_and(2**1) > 0) & (uvm2_signif < 10) |
-              (uvw1_quality_flag.bin_and(2**2) > 0) & (uvw1_signif < 10) |
-              (uvw2_quality_flag.bin_and(2**2) > 0) & (uvw2_signif < 10) |
               (uvm2_quality_flag.bin_and(2**2) > 0) & (uvm2_signif < 10) |
-              (uvw1_quality_flag.bin_and(2**5) > 0) & (uvw1_signif < 10) |
-              (uvw2_quality_flag.bin_and(2**5) > 0) & (uvw2_signif < 10) |
               (uvm2_quality_flag.bin_and(2**5) > 0) & (uvm2_signif < 10) |
-              (uvw1_quality_flag.bin_and(2**6) > 0) |
-              (uvw2_quality_flag.bin_and(2**6) > 0) |
               (uvm2_quality_flag.bin_and(2**6) > 0))
         )
 
@@ -627,10 +586,11 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
                        .where(*quality_flags)
                        .cte('quality_cte', materialized=True))
 
-        colour_cuts = (colour_absg,
-                       ((quality_cte.c.uvm2_ab - gmagab) < 2.25) |
-                       (((quality_cte.c.uvm2_ab - gmagab) < 6) &
-                        ((quality_cte.c.uvm2_ab - gmagab) < 5.57377 * (B_AB_R_AB) + 0.2049)))
+        colour_cuts = (
+            colour_absg,
+            ((quality_cte.c.uvm2_ab - gmagab) < 2.25) |
+            (((quality_cte.c.uvm2_ab - gmagab) < 6) &
+             ((quality_cte.c.uvm2_ab - gmagab) < 5.57377 * (B_AB_R_AB) + 0.2049)))
 
         query = (Catalog
                  .select(Catalog.catalogid,
@@ -649,13 +609,13 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
                          gmagab.alias('gmagab'),
                          BJ.r_est,
                          BJ.r_lo,
-                         UVOT_SSC_1.srcnum,
+                         quality_cte.c.srcid,
+                         quality_cte.c.obsid,
+                         quality_cte.c.radeg.alias('uvot_ra'),
+                         quality_cte.c.dedeg.alias('uvot_dec'),
+                         quality_cte.c.suvm2,
                          quality_cte.c.uvm2_ab.alias('uvm2_ab'),
-                         quality_cte.c.uvw2_ab.alias('uvw2_ab'),
-                         quality_cte.c.uvw1_ab.alias('uvw1_ab'),
-                         quality_cte.c.fuvw2.alias('fuvw2'),
-                         quality_cte.c.fuvm2.alias('fuvm2'),
-                         quality_cte.c.fuvw1.alias('fuvw1'))
+                         quality_cte.c.fuvm2.alias('fuvm2'))
                  .join(CatalogToTIC_v8)
                  .join(TIC_v8)
                  .join(Gaia_DR2)
@@ -680,7 +640,7 @@ class MWM_CB_UVEX4_Carton(BaseCarton):
         return query
 
 
-class MWM_CB_UVEX5_Carton(BaseCarton):
+class MWM_CB_UVEX5_Carton(MWM_CB_UVEX_BaseCarton):
     """MWM Compact Binaries UV excess 5.
 
     Definition:
@@ -739,9 +699,8 @@ class MWM_CB_UVEX5_Carton(BaseCarton):
     mapper = 'MWM'
     category = 'science'
     program = 'mwm_cb'
-    # mwm_cb_2x1 -> boss_dark_2x1
     instrument = 'BOSS'
-    cadence = 'dark_2x1'
+    cadence = None
     priority = 1400
 
     def setup_transaction(self):
@@ -784,7 +743,11 @@ class MWM_CB_UVEX5_Carton(BaseCarton):
                          GUVCat.fuv_mag,
                          GUVCat.nuv_magerr,
                          GUVCat.fuv_magerr,
-                         GMS.alias('GMS'))
+                         GMS.alias('GMS'),
+                         TwoMassPSC.ra,
+                         TwoMassPSC.decl,
+                         TwoMassPSC.h_m,
+                         TwoMassPSC.pts_key)
                  .join(CatalogToGUVCat)
                  .join(CatalogToTIC_v8,
                        on=(CatalogToGUVCat.catalogid == CatalogToTIC_v8.catalogid))
