@@ -304,10 +304,10 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
     # SDSS_APOGEE_AllStarMerge_r13(CatalogdbModel)--->'sdss_apogeeallstarmerge_r13'
 
     def build_query(self, version_id, query_region=None):
-        ra_hours_to_degrees = (360.0 / 24.0)
+        # ra_hours_to_degrees = (360.0 / 24.0)
 
-        ra = [0] * 40
-        dec = [0] * 40
+        ra = [0] * 35
+        dec = [0] * 35
         ra[1] = 11.83254;  dec[1] = 85.251     # noqa: E702, E241
         ra[2] = 47.39417;  dec[2] = 39.50294   # noqa: E702, E241
         ra[3] = 72.4084;   dec[3] = 63.603     # noqa: E702, E241
@@ -329,62 +329,47 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
         ra[19] = 240.575;  dec[19] = 28.09     # noqa: E702, E241
         ra[20] = 248.3458; dec[20] = -0.5336   # noqa: E702, E241
 
-        # M67_btx
-        ra[21] = 8.8695 * ra_hours_to_degrees; dec[21] = 11.8252  # noqa: E702, E241
+        # 172+58_btx
+        ra[21] = 159.0315; dec[21] = 43.7589  # noqa: E702, E241
 
-        # M35N2158_btx
-        ra[22] = 6.126 * ra_hours_to_degrees; dec[22] = 24.32512  # noqa: E702, E241
+        # 175+65_btx
+        ra[22] = 167.712; dec[22] = 39.8448  # noqa: E702, E241
 
-        # NGC2632_btx
-        ra[23] = 8.6685 * ra_hours_to_degrees; dec[23] = 19.6596  # noqa: E702, E241
+        # 232+66_btx
+        ra[23] = 170.031; dec[23] = 17.4326  # noqa: E702, E241
 
-        # M3-RV
-        ra[24] = 13.7531 * ra_hours_to_degrees; dec[24] = 27.6256  # noqa: E702, E241
+        # 209+73_btx
+        ra[24] = 174.207; dec[24] = 26.9809  # noqa: E702, E241
 
-        # BOOTES1
-        ra[25] = 13.9907 * ra_hours_to_degrees; dec[25] = 14.7  # noqa: E702, E241
+        # 248+68_btx
+        ra[25] = 174.642; dec[25] = 13.4369  # noqa: E702, E241
 
-        # URMINOR
-        ra[26] = 15.1933 * ra_hours_to_degrees; dec[26] = 67.11  # noqa: E702, E241
+        # 279+61_btx
+        ra[26] = 181.8465; dec[26] = 0.2  # noqa: E702, E241
 
-        # K2EB_C06_320+48_btx
-        ra[27] = 13.62 * ra_hours_to_degrees; dec[27] = -13.28  # noqa: E702, E241
+        # 246+77_btx
+        ra[27] = 181.9035; dec[27] = 19.9253  # noqa: E702, E241
 
-        # GD1-1
-        ra[28] = 8.4333 * ra_hours_to_degrees; dec[28] = 0.1666  # noqa: E702, E241
+        # 290+76_btx
+        ra[28] = 189.867; dec[28] = 14.0728  # noqa: E702, E241
 
-        # GD1-2
-        ra[29] = 8.7667 * ra_hours_to_degrees; dec[29] = 11.8833  # noqa: E702, E241
+        # 296+61_btx
+        ra[29] = 189.921; dec[29] = -1.3413  # noqa: E702, E241
 
-        # GD1-3
-        ra[30] = 9.25 * ra_hours_to_degrees; dec[30] = 22.3166  # noqa: E702, E241
+        # 120+67_btx
+        ra[30] = 194.2575; dec[30] = 49.9041  # noqa: E702, E241
 
-        # GD1-4
-        ra[31] = 10.9333 * ra_hours_to_degrees; dec[31] = 48.7333  # noqa: E702, E241
+        # 104+77_btx
+        ra[31] = 197.9385; dec[31] = 38.743  # noqa: E702, E241
 
-        # GD1-5
-        ra[32] = 14.4833 * ra_hours_to_degrees; dec[32] = 57.5166  # noqa: E702, E241
+        # 041+78_btx
+        ra[32] = 205.5945; dec[32] = 28.1849  # noqa: E702, E241
 
-        # ORPHAN-5
-        ra[33] = 9.4833 * ra_hours_to_degrees; dec[33] = 47.8666  # noqa: E702, E241
+        # 071+62_btx
+        ra[33] = 221.169; dec[33] = 41.0707  # noqa: E702, E241
 
-        # ORPHAN-4
-        ra[34] = 9.8167 * ra_hours_to_degrees; dec[34] = 38.0999  # noqa: E702, E241
-
-        # ORPHAN-3
-        ra[35] = 10.1167 * ra_hours_to_degrees; dec[35] = 26.0833  # noqa: E702, E241
-
-        # ORPHAN-2
-        ra[36] = 10.4167 * ra_hours_to_degrees; dec[36] = 15.0499  # noqa: E702, E241
-
-        # ORPHAN-1
-        ra[37] = 10.8833 * ra_hours_to_degrees; dec[37] = -3.0667  # noqa: E702, E241
-
-        # PAL5-2
-        ra[38] = 16.0333 * ra_hours_to_degrees; dec[38] = 6.1666  # noqa: E702, E241
-
-        # PAL5-1
-        ra[39] = 14.8333 * ra_hours_to_degrees; dec[39] = -7.1  # noqa: E702, E241
+        # 063+55_btx
+        ra[34] = 231.8415; dec[34] = 39.0758  # noqa: E702, E241
 
         ra_dec_condition = (
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[1], dec[1], 3) |
@@ -420,12 +405,12 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[31], dec[31], 3) |
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[32], dec[32], 3) |
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[33], dec[33], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[34], dec[34], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[35], dec[35], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[36], dec[36], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[37], dec[37], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[38], dec[38], 3) |
-            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[39], dec[39], 3))
+            peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[34], dec[34], 3))
+
+        catalog_regions = (Catalog
+                           .select(Catalog.catalogid, Catalog.ra, Catalog.dec)
+                           .where(ra_dec_condition)
+                           .cte('catalog_regions'))
 
 # We use *mwm_rv_long_condition to unpack the tuple mwm_rv_long_condition.
 # However, ra_dec_condition is not a tuple so it does not have a * in the front.
@@ -440,8 +425,11 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
                          SDSS_APOGEE_AllStarMerge_r13.h,
                          SDSS_APOGEE_AllStarMerge_r13.baseline,
                          SDSS_APOGEE_AllStarMerge_r13.fields)
+                 .join(catalog_regions,
+                       on=(Catalog.catalogid == catalog_regions.c.catalogid))
+                 .switch(Catalog)
                  .join(CatalogToTIC_v8,
-                       on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
+                       on=(catalog_regions.c.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8,
                        on=(CatalogToTIC_v8.target_id == TIC_v8.id))
                  .join(TwoMassPSC,
@@ -453,16 +441,16 @@ class MWM_RV_Long_Bplates_Carton(BaseCarton):
                  .where(CatalogToTIC_v8.version_id == version_id,
                         CatalogToTIC_v8.best >> True,
                         *mwm_rv_long_condition,
-                        ra_dec_condition,
-                        SDSS_APOGEE_AllStarMerge_r13.nvisits >= 6))
+                        SDSS_APOGEE_AllStarMerge_r13.nvisits >= 6)
+                 .with_cte(catalog_regions))
         # Below ra, dec and radius are in degrees
         # query_region[0] is ra of center of the region
         # query_region[1] is dec of center of the region
         # query_region[2] is radius of the region
         if query_region:
             query = (query
-                     .where(peewee.fn.q3c_radial_query(Catalog.ra,
-                                                       Catalog.dec,
+                     .where(peewee.fn.q3c_radial_query(catalog_regions.c.ra,
+                                                       catalog_regions.c.dec,
                                                        query_region[0],
                                                        query_region[1],
                                                        query_region[2])))
@@ -919,6 +907,11 @@ class MWM_RV_Short_Bplates_Carton(BaseCarton):
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[19], dec[19], 3) |
             peewee.fn.q3c_radial_query(Catalog.ra, Catalog.dec, ra[20], dec[20], 3))
 
+        catalog_regions = (Catalog
+                           .select(Catalog.catalogid, Catalog.ra, Catalog.dec)
+                           .where(ra_dec_condition)
+                           .cte('catalog_regions'))
+
 # We use *mwm_rv_short_condition to unpack the tuple mwm_rv_short_condition.
 # However, ra_dec_condition is not a tuple so it does not have a * in the front.
         query = (Catalog
@@ -928,6 +921,8 @@ class MWM_RV_Short_Bplates_Carton(BaseCarton):
                          Gaia_DR2.pmra.alias('gaia_dr2_pmra'),
                          Gaia_DR2.pmdec.alias('gaia_dr2_pmdec'),
                          TwoMassPSC.h_m.alias('twomass_h_m'))
+                 .join(catalog_regions,
+                       on=(Catalog.catalogid == catalog_regions.c.catalogid))
                  .join(CatalogToTIC_v8,
                        on=(Catalog.catalogid == CatalogToTIC_v8.catalogid))
                  .join(TIC_v8, on=(CatalogToTIC_v8.target_id == TIC_v8.id))
@@ -939,8 +934,8 @@ class MWM_RV_Short_Bplates_Carton(BaseCarton):
                  .where(CatalogToTIC_v8.version_id == version_id,
                         CatalogToTIC_v8.best >> True,
                         *mwm_rv_short_condition,
-                        ra_dec_condition,
-                        TwoMassPSC.h_m < 12.2))
+                        TwoMassPSC.h_m < 12.2)
+                 .with_cte(catalog_regions))
         # Below ra, dec and radius are in degrees
         # query_region[0] is ra of center of the region
         # query_region[1] is dec of center of the region
