@@ -19,7 +19,7 @@ import numpy
 import peewee
 import yaml
 from networkx.algorithms import shortest_path
-from peewee import SQL, Case, CompositeKey, Model, fn
+from peewee import SQL, Case, Model, fn
 
 from sdssdb.connection import PeeweeDatabaseConnection
 from sdssdb.utils.internals import get_row_count
@@ -56,7 +56,7 @@ class Version(peewee.Model):
 class Catalog(peewee.Model):
     """Model for the output table."""
 
-    catalogid = peewee.BigIntegerField(primary_key=True, null=False)
+    catalogid = peewee.BigIntegerField(index=True, null=False)
     iauname = peewee.TextField(null=True)
     ra = peewee.DoubleField(null=False)
     dec = peewee.DoubleField(null=False)
@@ -70,7 +70,7 @@ class Catalog(peewee.Model):
 class TempCatalog(Catalog):
     """Temporary output table."""
 
-    catalogid = peewee.BigIntegerField(index=True, primary_key=False)
+    catalogid = peewee.BigIntegerField(index=True, null=False)
     version_id = peewee.IntegerField(index=False)
 
     class Meta:
@@ -559,14 +559,13 @@ class XMatchPlanner(object):
         config = XMatchPlanner._read_config(config_file, plan)
 
         table_config = config.pop('tables', {}) or {}
-        exclude = config.pop('exclude', []) or []
+        exclude_nodes = config.pop('exclude_nodes', []) or []
 
         assert 'schema' in config, 'schema is required in configuration.'
         schema = config['schema']
 
         models = {model._meta.table_name: model for model in models
-                  if (model._meta.schema == schema and
-                      model._meta.table_name not in exclude)}
+                  if model._meta.schema == schema}
 
         xmatch_models = {}
         for table_name in table_config:
@@ -577,7 +576,8 @@ class XMatchPlanner(object):
                                                     **table_params)
 
         extra_nodes = [models[table_name] for table_name in models
-                       if table_name not in xmatch_models]
+                       if table_name not in xmatch_models and
+                       table_name not in exclude_nodes]
 
         config.update(kwargs)
 
@@ -796,7 +796,8 @@ class XMatchPlanner(object):
                         sg_ext.append((table_name, resolution, 1))
                 # Use table name as second sorting order to use alphabetic order in
                 # case of draw.
-                sg_ordered = list(zip(*sorted(sg_ext, key=lambda x: (x[2], x[1], x[0]))))[0]
+                sg_ordered = list(
+                    zip(*sorted(sg_ext, key=lambda x: (x[2], x[1], x[0]))))[0]
                 ordered_tables.extend(sg_ordered)
 
         self.log.info(f'processing order: {ordered_tables}')
@@ -1178,16 +1179,14 @@ class XMatchPlanner(object):
 
             catalogid = peewee.BigIntegerField(null=False, index=True)
             target_id = model_pk_class(null=False, index=True)
-            version_id = peewee.SmallIntegerField(null=False)
+            version_id = peewee.SmallIntegerField(null=False, index=True)
             distance = peewee.DoubleField(null=True)
-            best = peewee.BooleanField(null=False, index=True)
+            best = peewee.BooleanField(null=False)
 
             class Meta:
                 database = meta.database
                 schema = meta.schema
-                primary_key = CompositeKey('version_id', 'catalogid', 'target_id')
-                indexes = ((('version_id', 'target_id', 'best'), False),
-                           (('version_id', 'best'), False))
+                primary_key = False
 
         model_prefix = ''.join(x.capitalize() or '_'
                                for x in prefix.rstrip().split('_'))
@@ -1468,13 +1467,14 @@ class XMatchPlanner(object):
 
         # We only need to care about already linked targets if phase 1 run.
         if 1 in self._phases_run:
-            insert_query = (insert_query
-                            .where(~fn.EXISTS(
-                                rel_model
-                                .select(SQL('1'))
-                                .where((rel_model.version_id == self._version_id) &
-                                       ((rel_model.catalogid == xmatched.c.catalogid) |
-                                        (rel_model.target_id == xmatched.c.target_id))))))
+            insert_query = (
+                insert_query .where(
+                    ~fn.EXISTS(
+                        rel_model .select(
+                            SQL('1')) .where(
+                            (rel_model.version_id == self._version_id) & (
+                                (rel_model.catalogid == xmatched.c.catalogid) | (
+                                    rel_model.target_id == xmatched.c.target_id))))))
 
         with Timer() as timer:
 
@@ -1523,7 +1523,6 @@ class XMatchPlanner(object):
 
         if 3 in xmatch.skip_phases:
             self.log.warning('Skipping due to configuration.')
-            self._analyze(rel_model, catalog=True)
             return
 
         table_name = meta.table_name
@@ -1541,11 +1540,13 @@ class XMatchPlanner(object):
                      .where(self._get_sample_where(model_ra, model_dec)))
 
         if 1 in self._phases_run or 2 in self._phases_run:
-            unmatched = (unmatched
-                         .where(~fn.EXISTS(rel_model
-                                           .select(SQL('1'))
-                                           .where(rel_model.version_id == self._version_id,
-                                                  rel_model.target_id == model_pk))))
+            unmatched = (
+                unmatched .where(
+                    ~fn.EXISTS(
+                        rel_model .select(
+                            SQL('1')) .where(
+                            rel_model.version_id == self._version_id,
+                            rel_model.target_id == model_pk))))
 
         if xmatch.has_missing_coordinates:
             unmatched = unmatched.where(model_ra.is_null(False),
@@ -1576,8 +1577,8 @@ class XMatchPlanner(object):
                 unmatched.create_table(temp_table, temporary=True)
 
                 # Analyze the temporary table to gather stats.
-                self.log.debug('Running ANALYZE on temporary table.')
-                self.database.execute_sql(f'ANALYZE "{temp_table}";')
+                # self.log.debug('Running ANALYZE on temporary table.')
+                # self.database.execute_sql(f'ANALYZE "{temp_table}";')
 
                 # 2. Copy data from temporary table to relational table. Add
                 #    catalogid at this point.
@@ -1645,12 +1646,14 @@ class XMatchPlanner(object):
 
         self._phases_run.add(3)
 
-        if n_rows > 0.2 * self._temp_count:  # Cluster if we have added > 20% new rows.
+        if n_rows > 0.5 * self._temp_count:  # Cluster if we have added > 50% new rows.
             self.log.debug(f'Running CLUSTER on {self._temp_table} with q3c index.')
             self.database.execute_sql(f'CLUSTER {self._temp_table} '
                                       f'using {self._temp_table}_q3c_idx;')
+            self.log.debug(f'Running ANALYZE on {self._temp_table}.')
+            self.database.execute_sql(f'ANALYZE {self._temp_table};')
 
-        self._analyze(rel_model, catalog=True)
+        self._analyze(rel_model, catalog=False)
 
     def _load_output_table(self, keep_temp=False):
         """Copies the temporary table to the real output table."""
@@ -1747,7 +1750,7 @@ class XMatchPlanner(object):
         """Removes duplicates from LS8 queries."""
 
         return ~(((model.release == 8000) & (model.dec > 32.375) &
-                 (model.ra > 100.) & (model.ra < 300.)) |
+                  (model.ra > 100.) & (model.ra < 300.)) |
                  ((model.release == 8001) & (model.dec < 32.375)))
 
     def show_join_paths(self):
